@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2019 Kevin De Bruycker and Tim Krappitz
+# Copyright (c) 2019-2021 Kevin De Bruycker and Tim Krappitz
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -35,6 +35,55 @@ import progressbar
 
 class Polymer:
 
+    @staticmethod
+    def getRUCombinations(minRepeatingUnits, maxRepeatingUnits, monomersAmount: int):
+        if type(minRepeatingUnits) == list:
+            if type(maxRepeatingUnits) == list:
+                # first check that the length of the list is valid
+                if not len(minRepeatingUnits) == len(maxRepeatingUnits) == monomersAmount:
+                    sys.exit("The length of minRepeatingUnits or maxRepeatingUnits does not match the amount of monomers!\nEither provide a max for each monomer or 1 global max for the sum of the repeating units")
+                RUCombinations = [(a,) for a in range(minRepeatingUnits[0], maxRepeatingUnits[0] + 1)]
+                for i in range(1, monomersAmount):
+                    RUCombinations = [(*a, b) for a in RUCombinations for b in
+                                      range(minRepeatingUnits[i], maxRepeatingUnits[i] + 1)]
+                return RUCombinations
+            elif type(maxRepeatingUnits) == int:
+                # first check that the length of the list is valid
+                if not len(minRepeatingUnits) == monomersAmount:
+                    sys.exit("The length of minRepeatingUnits does not match the amount of monomers!\nEither provide a max for each monomer or 1 global max for the sum of the repeating units")
+                RUCombinations = [(a,) for a in range(minRepeatingUnits[0], maxRepeatingUnits + 1)]
+                for i in range(1, monomersAmount):
+                    RUCombinations = [(*a, b) for a in RUCombinations for b in
+                                      range(minRepeatingUnits[i], maxRepeatingUnits + 1 - sum(a))]
+                return RUCombinations
+            else:
+                sys.exit("maxRepeatingUnits must be either int or list.")
+        elif type(minRepeatingUnits) == int:
+            # in this case, only a global minimum is given, so all combinations have to be made from 0
+            # the ones with a sum smaller than minRepeatingUnits will be filtered later
+            if type(maxRepeatingUnits) == list:
+                # first check that the length of the list is valid
+                if not len(maxRepeatingUnits) == monomersAmount:
+                    sys.exit("The length of maxRepeatingUnits does not match the amount of monomers!\nEither provide a max for each monomer or 1 global max for the sum of the repeating units")
+                RUCombinations = [(a,) for a in range(maxRepeatingUnits[0] + 1)]
+                for i in range(1, monomersAmount):
+                    RUCombinations = [(*a, b) for a in RUCombinations for b in range(maxRepeatingUnits[i] + 1)]
+                RUCombinations = [RUCombination for RUCombination in RUCombinations if
+                                  sum(RUCombination) >= minRepeatingUnits]
+                return None if len(RUCombinations) == 0 else RUCombinations
+            elif type(maxRepeatingUnits) == int:
+                # most simple case: every monomer from 0 to the maxRepeatingUnits minus what's already there from previous iterations
+                RUCombinations = [(a,) for a in range(maxRepeatingUnits + 1)]
+                for i in range(1, monomersAmount):
+                    RUCombinations = [(*a, b) for a in RUCombinations for b in range(maxRepeatingUnits + 1 - sum(a))]
+                RUCombinations = [RUCombination for RUCombination in RUCombinations if
+                                  sum(RUCombination) >= minRepeatingUnits]
+                return None if len(RUCombinations) == 0 else RUCombinations
+            else:
+                sys.exit("maxRepeatingUnits must be either int or list.")
+        else:
+            sys.exit("minRepeatingUnits must be either int or list.")
+
     def __init__(self,
                  endgroupPairs,
                  monomers,
@@ -136,42 +185,13 @@ class Polymer:
                 minRepeatingUnits = 0
                 print("--> minimum sum of repeating units set to 0 since it cannot be estimated")
         # Now build all the combinations of repeating units within the limits of the given (or estimated) parameters
-        if type(minRepeatingUnits) == int:
-            # in this case, only a global minimum is given, so all combinations have to be made from 0
-            # the ones with a sum smaller than minRepeatingUnits will be filtered later
-            if type(maxRepeatingUnits) == int:
-                # most simple case: every monomer from 0 to the same maxRepeatingUnits
-                # combinations with a too high sum will be filtered later
-                RUCombinations = list(itertools.product(range(maxRepeatingUnits + 1), repeat=len(monomers)))
-            elif type(maxRepeatingUnits) == list:
-                # first check that the length of the list is valid
-                if len(maxRepeatingUnits) != len(monomers):
-                    sys.exit("The length of maxRepeatingUnits does not match the amount of monomers!\nEither provide a max for each monomer or 1 global max for the sum of the repeating units")
-                # similar as before but instead of the same range repeated for "amount of monomer times", here different ranges are used
-                RUCombinations = list(itertools.product(*(range(max + 1) for max in maxRepeatingUnits)))
-                maxRepeatingUnits = sum(maxRepeatingUnits)
-            else:
-                sys.exit("Something went wrong when interpreting maxRepeatingUnits\nCheck your input")
-        elif type(minRepeatingUnits) == list:
-            # first check that the length of the list is valid
-            if len(minRepeatingUnits) != len(monomers):
-                sys.exit("The length of minRepeatingUnits does not match the amount of monomers!\nEither provide a min for each monomer or 1 global min for the sum of the repeating units")
-            if type(maxRepeatingUnits) == int:
-                # the same formula as earlier can be used, but now the list comprehension will be over the minimum with a constant maximum
-                # again, the combinations with a too high sum will be filtered later
-                RUCombinations = list(itertools.product(*(range(min, maxRepeatingUnits + 1) for min in minRepeatingUnits)))
-            elif type(maxRepeatingUnits) == list:
-                # first check that the length of the list is valid
-                if len(maxRepeatingUnits) != len(monomers):
-                    sys.exit("The length of maxRepeatingUnits does not match the amount of monomers!\nEither provide a max for each monomer or 1 global max for the sum of the repeating units")
-                # now both the minima and maxima are fixed, both min and max lists are combined using zip
-                RUCombinations = list(itertools.product(*(range(min, max + 1) for min, max in list(zip(minRepeatingUnits, maxRepeatingUnits)))))
-                maxRepeatingUnits = sum(maxRepeatingUnits)
-            else:
-                sys.exit("Something went wrong when interpreting maxRepeatingUnits\nCheck your input")
+        RUCombinations = self.getRUCombinations(minRepeatingUnits, maxRepeatingUnits, len(monomers))
+        if RUCombinations is None:
+            sys.exit("Failed to build a matrix of possible combinations of monomers based on \nthe provided combination of mzRange, minRepeatingUnits and maxRepeatingUnits.\nPlease check the parameters.")
+        if type(minRepeatingUnits) == list:
             minRepeatingUnits = sum(minRepeatingUnits)
-        else:
-            sys.exit("Something went wrong when interpreting minRepeatingUnits\nCheck your input")
+        if type(maxRepeatingUnits) == list:
+            maxRepeatingUnits = sum(maxRepeatingUnits)
         if minRepeatingUnits > maxRepeatingUnits:
             sys.exit("With the current parameters, minRepeatingUnits is higher than maxRepeatingUnits, this can't be correct...")
         # Finally, combine the combinations of repeating units with the different indices of the different end groups
